﻿
Imports System.Linq.Expressions
Imports Microsoft.EntityFrameworkCore
Imports UoWTestApp.Core.Repositories
Imports UoWTestApp.Interfaces

Namespace Persistence.Repositories
    Public Class Repository(Of TEntity As Class)
        Implements IRepository(Of TEntity)
        Implements IDisposable

        Private _entities As New Object
        Private ReadOnly _disposeContext As Boolean = True

        Friend ReadOnly Property ContextInternal As UoWTestAppModelCore
            Get
                Return CType(Context, UoWTestAppModelCore)
            End Get
        End Property

        Public Property Context() As Object Implements IRepository(Of TEntity).Context

            Get
                Return _entities
            End Get
            Set
                _entities = Value
            End Set
        End Property

        Protected Friend Sub New(Optional _Context As UoWTestAppModelCore = Nothing, Optional tracking As Boolean = False)

            If _Context IsNot Nothing Then
                Context = _Context
                _disposeContext = False
            Else
                Context = New UoWTestAppModelCore()
            End If

            ContextInternal.ChangeTracker.AutoDetectChangesEnabled = tracking
            If tracking Then
                ContextInternal.ChangeTracker.QueryTrackingBehavior = QueryTrackingBehavior.TrackAll
            Else
                ContextInternal.ChangeTracker.QueryTrackingBehavior = QueryTrackingBehavior.NoTracking
            End If
        End Sub

        Public Function [Get](ByVal id As Guid) As TEntity Implements IRepository(Of TEntity).Get
            Return Context.[Set](Of TEntity)().Find(id)
        End Function

        Public Function GetAll(Optional IncludeDeleted As Boolean = False) As IQueryable(Of TEntity) Implements IRepository(Of TEntity).GetAll
            Dim Query As IQueryable(Of TEntity) = Context.Set(Of TEntity)
            If Not IncludeDeleted Then
                If GetType(IDelete).IsAssignableFrom(GetType(TEntity)) Then
                    Query = Query.Where(Function(x) DirectCast(x, IDelete).Deleted = False)
                End If
            End If
            Return Query
        End Function

        Public Function Find(ByVal predicate As Expression(Of Func(Of TEntity, Boolean))) As IEnumerable(Of TEntity) Implements IRepository(Of TEntity).Find
            Return Context.[Set](Of TEntity)().Where(predicate)
        End Function

        Public Function SingleOrDefault(ByVal predicate As Expression(Of Func(Of TEntity, Boolean))) As TEntity Implements IRepository(Of TEntity).SingleOrDefault
            Return Context.[Set](Of TEntity)().SingleOrDefault(predicate)
        End Function

        Public Sub Add(ByVal entity As TEntity) Implements IRepository(Of TEntity).Add
            Context.[Set](Of TEntity)().Add(entity)
        End Sub

        Public Sub AddRange(ByVal entities As IEnumerable(Of TEntity)) Implements IRepository(Of TEntity).AddRange
            Context.[Set](Of TEntity)().AddRange(entities)
        End Sub

        Public Sub Remove(ByVal entity As TEntity) Implements IRepository(Of TEntity).Remove
            Context.[Set](Of TEntity)().Remove(entity)
        End Sub

        Public Sub RemoveRange(ByVal entities As IEnumerable(Of TEntity)) Implements IRepository(Of TEntity).RemoveRange
            Context.[Set](Of TEntity)().RemoveRange(entities)
        End Sub

#Region "IDisposable Support"
        Private disposedValue As Boolean ' To detect redundant calls

        ' IDisposable
        Protected Overridable Sub Dispose(disposing As Boolean)
            If Not disposedValue Then
                If disposing Then
                    ' TODO: dispose managed state (managed objects).
                End If

                ' TODO: free unmanaged resources (unmanaged objects) and override Finalize() below.
                ' TODO: set large fields to null.
            End If
            disposedValue = True
        End Sub

        ' TODO: override Finalize() only if Dispose(disposing As Boolean) above has code to free unmanaged resources.
        'Protected Overrides Sub Finalize()
        '    ' Do not change this code.  Put cleanup code in Dispose(disposing As Boolean) above.
        '    Dispose(False)
        '    MyBase.Finalize()
        'End Sub

        ' This code added by Visual Basic to correctly implement the disposable pattern.
        Public Sub Dispose() Implements IDisposable.Dispose
            ' Do not change this code.  Put cleanup code in Dispose(disposing As Boolean) above.
            Dispose(True)
            ' TODO: uncomment the following line if Finalize() is overridden above.
            ' GC.SuppressFinalize(Me)

            If _disposeContext Then
                Debug.WriteLine("Disposing GenericRepository...")
                ContextInternal.Dispose()
                Debug.WriteLine("GenericRepository Disposed")
            Else
                Debug.WriteLine("Do not Dispose Context of GenericRepository because its created from another place!!")
            End If

        End Sub
#End Region


    End Class
End Namespace